package com.weface.common.utils;


import com.alibaba.fastjson.JSON;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import java.io.BufferedReader;
import java.io.DataOutputStream;
import java.io.InputStreamReader;
import java.net.HttpURLConnection;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.util.List;
import java.util.Map;

/**
 * http 工具类
 */
@Slf4j
public class HttpUtil {

    public static String post(String requestUrl, String accessToken, String params) throws Exception {
        String contentType = "application/x-www-form-urlencoded";
        return HttpUtil.post(requestUrl, accessToken, contentType, params);
    }

    public static String post(String requestUrl, String accessToken, String contentType, String params) throws Exception {
        String encoding = "UTF-8";
        if (requestUrl.contains("nlp")) {
            encoding = "GBK";
        }
        return HttpUtil.post(requestUrl, accessToken, contentType, params, encoding);
    }

    public static String post(String requestUrl, String accessToken, String contentType, String params, String encoding) throws Exception {
        String url = requestUrl + "?access_token=" + accessToken;
        return HttpUtil.postGeneralUrl(url, contentType, params, encoding);
    }

    public static String postGeneralUrl(String generalUrl, String contentType, String params, String encoding) throws Exception {
        return HttpUtil.postGeneralUrl(generalUrl, contentType, params, encoding, null);
    }

    public static String postGeneralUrl(String generalUrl, String contentType, String params, String encoding, String token) throws Exception {
        URL url = new URL(generalUrl);
        // 打开和URL之间的连接
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("POST");
        // 设置通用的请求属性
        connection.setRequestProperty("Content-Type", contentType);
        connection.setRequestProperty("Connection", "Keep-Alive");
        if (StringUtils.isNotEmpty(token)) {
            connection.setRequestProperty("token", token);
        }
        connection.setUseCaches(false);
        connection.setDoOutput(true);
        connection.setDoInput(true);
        // 得到请求的输出流对象
        DataOutputStream out = new DataOutputStream(connection.getOutputStream());
        out.write(params.getBytes(StandardCharsets.UTF_8));
        out.flush();
        out.close();

        // 建立实际的连接
        connection.connect();
        // 获取所有响应头字段
        Map<String, List<String>> headers = connection.getHeaderFields();
        // 遍历所有的响应头字段
//        for (String key : headers.keySet()) {
//            log.info(key + "--->" + headers.get(key));
//        }
        // 定义 BufferedReader输入流来读取URL的响应
//        BufferedReader in = null;
        BufferedReader in = new BufferedReader(new InputStreamReader(connection.getInputStream(), encoding));
        String result = "";
        String getLine;
        while ((getLine = in.readLine()) != null) {
            result += getLine;
        }
        in.close();
        return result;
    }

    public static String getGeneralUrl(String generalUrl, String contentType, String params, String encoding)
            throws Exception {
        return HttpUtil.getGeneralUrl(generalUrl, contentType, params, encoding, null);
    }

    public static String getGeneralUrl(String generalUrl, String contentType, String params, String encoding, String token)
            throws Exception {
        URL url = new URL(generalUrl);
        // 打开和URL之间的连接
        HttpURLConnection connection = (HttpURLConnection) url.openConnection();
        connection.setRequestMethod("GET");
        // 设置通用的请求属性
        connection.setRequestProperty("Content-Type", contentType);
        connection.setRequestProperty("Connection", "Keep-Alive");
        if (StringUtils.isNotEmpty(token)) {
            connection.setRequestProperty("token", token);
        }
        connection.setUseCaches(false);
        connection.setDoOutput(true);
        connection.setDoInput(true);
        // 得到请求的输出流对象
        DataOutputStream out = new DataOutputStream(connection.getOutputStream());
        out.write(params.getBytes(encoding));
        out.flush();
        out.close();

        // 建立实际的连接
        connection.connect();
        // 获取所有响应头字段
        Map<String, List<String>> headers = connection.getHeaderFields();
        // 遍历所有的响应头字段
        for (String key : headers.keySet()) {
            log.info(key + "--->" + headers.get(key));
        }
        // 定义 BufferedReader输入流来读取URL的响应
        BufferedReader in = null;
        in = new BufferedReader(
                new InputStreamReader(connection.getInputStream(), encoding));
        String result = "";
        String getLine;
        while ((getLine = in.readLine()) != null) {
            result += getLine;
        }
        in.close();
        log.info("result:" + result);
        return result;
    }

    public static boolean isJson2(String str) {
        boolean flag = false;
        try {
            Object parse = JSON.parse(str);
            flag = true;
        } catch (Exception e) {
            e.printStackTrace();
            flag = false;
        }
        return flag;
    }

//    public static String postGeTui(String url, Object obj, String token) {
//        CloseableHttpClient httpClient = HttpClients.createDefault();
//        HttpPost httpPost = new HttpPost(url);
//        httpPost.setHeader("token", token);
//        httpPost.setHeader("Content-Type", "application/json");
//        httpPost.setHeader("Accept", "application/json");
//        StringEntity stringEntity = new StringEntity(JSONObject.toJSONString(obj), "UTF-8");
//        System.out.println(stringEntity);
//        httpPost.setEntity(stringEntity);
//        CloseableHttpResponse response = null;
//        try {
//            response = httpClient.execute(httpPost);
//            StatusLine status = response.getStatusLine();
//            int state = status.getStatusCode();
//            if (state == HttpStatus.SC_OK) {
//                HttpEntity responseEntity = response.getEntity();
//                return EntityUtils.toString(responseEntity);
//            } else {
//                log.error("请求返回:" + state + "(" + url + ")");
//            }
//        } catch (IOException e) {
//            e.printStackTrace();
//        } finally {
//            IoUtil.close(response);
//            IoUtil.close(httpClient);
//        }
//
//        return null;
//    }
}
