package com.weface.component;

import cn.hutool.core.lang.Console;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.http.HttpRequest;
import cn.hutool.json.JSONUtil;
import com.alibaba.fastjson.JSONObject;

import com.weface.common.utils.Base64Util;
import com.weface.common.utils.HttpUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.ArrayUtils;
import org.apache.commons.lang.StringUtils;

import java.util.HashMap;
import java.util.Map;

/**
 * @CreateUser: Administrator
 * @CreateTime: 2021/11/2
 */
@Slf4j
public class GeTuiUtils {
    //请求前缀
    public static final String GETUIBASEURL = "https://restapi.getui.com/v2/";
    public static final String APPID = "CYol79N33N71BV6dcjrqj3";
    public static final String BASEURL = "https://openapi-gi.getui.com/v2/";
    public static final String ENCODING = "UTF-8";
    public static final String CONTENTTYPE = "application/json;charset=utf-8";
    public static final String APPKEY = "rdLx5zumRK7oEme8MheAh8";
    public static final String MASTERSECRET = "mekLZ4frLu7RHtKsN9mQn";

    /**
     * 获取个像token
     *
     * @return JSONObject
     */
    public static String getAuth() {
        // 获取token地址
        log.info("开始获取个像token");
        String getAccessTokenUrl = BASEURL + APPID + "/auth_sign";
        Long timestamp = System.currentTimeMillis();
        try {
            Map<String, Object> map = new HashMap<>();
            map.put("sign", Base64Util.getSign(APPKEY, timestamp, MASTERSECRET));
            map.put("timestamp", timestamp);
            String result = HttpUtil.postGeneralUrl(getAccessTokenUrl, CONTENTTYPE, JSONObject.toJSONString(map), ENCODING);
            if (HttpUtil.isJson2(result)) {
                JSONObject jsonObject = JSONObject.parseObject(result);
                if ("true".equals(jsonObject.getString("result"))) {
                    Console.log(jsonObject);
                    return jsonObject.getString("authtoken");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取个像token失败!");
        }
        return null;
    }

    /**
     * 获取个推token
     *
     * @return JSONObject
     */
    public static JSONObject getGeTuiAuth() {
        // 获取token地址
        log.info("开始获取个推token");
        String getAccessTokenUrl = GETUIBASEURL + APPID + "/auth";
        Long timestamp = System.currentTimeMillis();
        try {
            Map<String, Object> map = new HashMap<>();
            map.put("sign", Base64Util.getSign(APPKEY, timestamp, MASTERSECRET));
            map.put("timestamp", timestamp);
            map.put("appkey", APPKEY);
            String result = HttpUtil.postGeneralUrl(getAccessTokenUrl, CONTENTTYPE, JSONObject.toJSONString(map), ENCODING);
            if (HttpUtil.isJson2(result)) {
                JSONObject jsonObject = JSONObject.parseObject(result);
                if (jsonObject.containsKey("code") && jsonObject.getInteger("code") == 0) {
                    return jsonObject.getJSONObject("data");
                }
            }
        } catch (Exception e) {
            e.printStackTrace();
            log.error("获取个推token失败!");
        }
        return null;
    }

    /**
     * 根据giUid查询用户相关标签code
     *
     * @param userIdList
     * @param token
     * @return
     * @throws Exception
     */
    public static String queryTag(String[] userIdList, String token) throws Exception {
        //请求url
        String getUsersCidUrl = "https://openapi-gi.getui.com/v3/" + APPID + "/query_tag";
        //将token以及用户ID封装调用画像查询接口
        Map<String, Object> map = new HashMap<>();
        map.put("userIdList", userIdList);
        map.put("token", token);
        //发送post请求拿到当前用户的所有图像
        return HttpUtil.postGeneralUrl(getUsersCidUrl, CONTENTTYPE, JSONObject.toJSONString(map), ENCODING);
    }

    /**
     * 执行别名单推
     *
     * @param alias           用户别名
     * @param messageTemplate 消息模板
     * @param token           token
     * @return 消息推送结果
     * @throws Exception
     */
    public static String singlePush(String[] alias, MessageTemplate messageTemplate, String token) throws Exception {
        Map<String, Object> map = convertBeanToMap(alias, messageTemplate);
        //请求url
        String url = GETUIBASEURL + APPID + "/push/single/alias";
        HttpRequest request = cn.hutool.http.HttpUtil.createPost(url);
        request.header("token", token);
        request.body(JSONUtil.toJsonStr(map));
        log.info("执行别名单推");
        return request.execute().body();
    }


    /**
     * 创建消息拿到任务id
     *
     * @param messageTemplate 消息模板
     * @param token           token
     * @return 任务id
     * @throws Exception
     */
    public static String getTaskId(MessageTemplate messageTemplate, String token) throws Exception {
        Map<String, Object> map = convertBeanToMap(null, messageTemplate);
        //请求url
        String url = GETUIBASEURL + APPID + "/push/list/message";
        //创建post请求
        HttpRequest request = cn.hutool.http.HttpUtil.createPost(url);
        //设置请求头token
        request.header("token", token);
        //设置请求体数据
        request.body(JSONUtil.toJsonStr(map));
        ////创建消息 获取taskid
        log.info("获取任务id");
        String body = request.execute().body();
        JSONObject jsonObject = JSONObject.parseObject(body);
        return (jsonObject.containsKey("code") && jsonObject.getInteger("code") == 0) ? jsonObject.getJSONObject("data").getString("taskid") : body;
    }

    /**
     * 实体类封装为map
     *
     * @param alias           别名
     * @param messageTemplate 消息模板
     * @return map对象
     */
    public static Map<String, Object> convertBeanToMap(String[] alias, MessageTemplate messageTemplate) {
        if (ArrayUtils.isEmpty(alias) || messageTemplate == null) return null;
        Map<String, Object> map = new HashMap<>();
        //request_id 唯一不重复
        map.put("request_id", System.currentTimeMillis() + RandomUtil.randomString(15));

        //任务组名
        String group_name = messageTemplate.getGroup_name();
        if (StringUtils.isNotBlank(group_name)) {
            map.put("group_name", group_name);
        }

        Map<String, Object> audienceMap = new HashMap<>();
        audienceMap.put("alias", alias);
        map.put("audience", audienceMap);

        //消息离线时间设置，单位毫秒，-1表示不设离线，-1 ～ 3 * 24 * 3600 * 1000(3天)之间
        Long ttl = messageTemplate.getTtl();
        if (ttl != null) {
            Map<String, Object> settingsMap = new HashMap<>();
            settingsMap.put("ttl", ttl);
            map.put("settings", settingsMap);
        }

        //消息模板
        Map<String, Object> pushMessageMap = new HashMap<>();
        Map<String, Object> notificationMap = new HashMap<>();
        notificationMap.put("title", messageTemplate.getTitle());
        notificationMap.put("body", messageTemplate.getBody());
        notificationMap.put("click_type", messageTemplate.getClick_type());
        switch (messageTemplate.getClick_type()) {
            case "intent":
                notificationMap.put("intent", messageTemplate.getIntent());
            case "url":
                notificationMap.put("url", messageTemplate.getUrl());
        }

        pushMessageMap.put("notification", notificationMap);
        map.put("push_message", pushMessageMap);
        return map;
    }

    /**
     * 执行别名批量推送
     *
     * @param map   body参数
     * @param token token
     * @return 执行结果
     */
    public static String listPushByAlias(Map<String, Object> map, String token) {
        //请求url
        String url = GETUIBASEURL + APPID + "/push/list/alias";
        //创建post请求
        HttpRequest request = cn.hutool.http.HttpUtil.createPost(url);
        //设置请求头token
        request.header("token", token);
        //设置请求体数据
        request.body(JSONUtil.toJsonStr(map));
        //执行请求
        log.info("执行别名批量推送");
        return request.execute().body();
    }
}
