package com.weface.serviceimpl;

import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.weface.dao.TUserTagDao;
import com.weface.dto.MenuTagsForm;
import com.weface.entity.TUserTagEntity;
import com.weface.service.TUserTagService;
import com.weface.common.utils.PageUtils;
import com.weface.common.utils.Query;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Service;

import java.util.*;


@Service("tUserTagService")
public class TUserTagServiceImpl extends ServiceImpl<TUserTagDao, TUserTagEntity> implements TUserTagService {


    /**
     * 查询用户和标签信息
     *
     * @param params 查询参数
     * @return 用户和标签数据分页
     */
    @Override
    public PageUtils getUserAndTags(Map<String, Object> params) {
        IPage<Object> page = new Query<>().getPage(params);
        String giUid = (String) params.get("giUid");
        String tag = (String) params.get("tag");
        IPage<TUserTagEntity> userAndTagsByUid = this.baseMapper.getUserAndTags(new Page<TUserTagEntity>(page.getCurrent(), page.getSize()), giUid, tag);
        userAndTagsByUid.setTotal(userAndTagsByUid.getRecords().size());
        return new PageUtils(userAndTagsByUid);
    }

    /**
     * 使用limit分页查询
     *
     * @param pageRecord 起始值
     * @param limit      每次查询数量
     * @return gid
     */
    @Override
    public List<String> findGidByLimit(int pageRecord, int limit) {
        return this.baseMapper.findGidByLimit(pageRecord, limit);
    }

    /**
     * 查询当天修改的用户gid
     *
     * @return
     */
    @Override
    public List<String> findTodayGid() {
        return this.baseMapper.findTodayGid();
    }

    /**
     * 根据标签查询用户gid
     *
     * @param tags 标签完全匹配
     * @return gid
     */
    @Override
    public List<String> getGidByTag(String[] tags) {
        return this.baseMapper.getGidByTag(Arrays.asList(tags));
    }

    /**
     * 根据gid查询用户标签
     *
     * @param userIdList gid
     * @return 标签信息
     */
    @Override
    public Map<String, Object> getTagByGid(String[] userIdList) {
        List<TUserTagEntity> userByUid = this.baseMapper.getTagByGid(Arrays.asList(userIdList));
        List<Map<String, Object>> list = new ArrayList<>(userByUid.size());
        for (TUserTagEntity tUserTagEntity : userByUid) {
            Map<String, Object> map = new HashMap<>();
            List<MenuTagsForm> objects = new ArrayList<>();
            tUserTagEntity.getList().forEach(item -> {
                MenuTagsForm menuTagsForm = new MenuTagsForm();
                BeanUtils.copyProperties(item, menuTagsForm);
                objects.add(menuTagsForm);
            });
            map.put("userId", tUserTagEntity.getGid());
            map.put("tags", objects);
            list.add(map);
        }
        Map<String, Object> map = new HashMap<>(2);
        map.put("error_code", 0);
        map.put("userTag", list);
        return map;
    }

    @Override
    public List<TUserTagEntity> getAllImport() {
        return this.baseMapper.selectAllUserForTags();
    }

}