package com.weface.task;

import com.weface.component.MenuService;
import com.weface.entity.UserMenusEntity;
import com.weface.service.TUserTagService;
import com.weface.service.UserMenusService;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.CollectionUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.concurrent.CountDownLatch;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

/**
 * @CreateUser: Administrator
 * @CreateTime: 2021/11/4
 */
@Component
@Slf4j
@EnableScheduling
public class UserTagsTask {

    @Autowired
    private MenuService menuService;
    @Autowired
    private TUserTagService tUserTagService;
    @Autowired
    private UserMenusService userMenusService;

    @Scheduled(cron = "0 0 0 1 * ?")
    public void updateUserTags() {
        try {
            List<String> todayGid = tUserTagService.findTodayGid();
            int size = todayGid.size();
            int limit = 1000;
            List<UserMenusEntity> userMenusList = new ArrayList<>();
            int totalPage = (size % limit == 0) ? (size / limit) : (size / limit + 1);
            CountDownLatch latch = new CountDownLatch(totalPage);
            ExecutorService pool = Executors.newFixedThreadPool(totalPage);
            //收集中间表数据
            for (int i = 0; i < totalPage; i++) {
                int finalI = i;
                Runnable runnable = () -> {
                    List<String> list = new ArrayList<>();
                    if (finalI == totalPage - 1) {
                        list = todayGid.subList(finalI * limit, size);
                    } else {
                        list = todayGid.subList(finalI * limit, (finalI + 1) * limit);
                    }
                    if (CollectionUtils.isNotEmpty(list)) {
                        String[] strings = list.toArray(new String[0]);
                        Map<String, Object> android = menuService.getUserTags(strings);
                        List<UserMenusEntity> tagUser = menuService.getTagUser(android);
                        if (CollectionUtils.isNotEmpty(tagUser)) {
                            userMenusList.addAll(tagUser);
                        }
                    }
                    latch.countDown();
                };
                pool.execute(runnable);
            }
            latch.await();
            userMenusService.batchInsert(userMenusList);
        } catch (InterruptedException e) {
            e.printStackTrace();
            log.error("执行定时更新标签任务失败!");
        }
    }

//    @Scheduled(cron = "* * * * * ?")
//    public void updateUser() {
//        System.out.println("你好" + System.currentTimeMillis());
//    }
}
